using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using Vevo.Shared.DataAccess.Implementation;
using Vevo.Shared.Utilities;

namespace Vevo.Shared.DataAccess
{
    public enum SearchFilterType
    {
        None,
        Text,
        Boolean,
        ValueRange,
        Date
    }


    public static class SearchFilterHelper
    {
        private static string CreateCommandNoPrefix(
            SearchFilterType filterType,
            string fieldName,
            string sqlField,
            string value1,
            string value2,
            ArrayList dbParametersToAppend
            )
        {
            string command = String.Empty;
            bool boolValue;
            bool requireAnd = false;
            if (String.IsNullOrEmpty( sqlField ))
                sqlField = fieldName;

            switch (filterType)
            {
                case SearchFilterType.Text:
                    if ( value1 != null && value1.Trim() != String.Empty )
                    {
                        command = String.Format( " {0} LIKE @{1} ", sqlField, fieldName );
                        dbParametersToAppend.Add(
                            DataAccess.CreateParameterString( "%" + ReplaceSpecialCharecter( value1.Trim() ) + "%" ) );
                    }
                    break;

                case SearchFilterType.Boolean:
                    if (Boolean.TryParse( value1, out boolValue ))
                    {
                        command = String.Format( " {0} = @{1} ", sqlField, fieldName );
                        dbParametersToAppend.Add( DataAccess.CreateParameterBool( boolValue ) );
                    }
                    break;

                case SearchFilterType.ValueRange:
                    if (value1 != null && value1.Trim() != String.Empty)
                    {
                        command += String.Format( " {0} >= @{1} ", sqlField, fieldName );
                        dbParametersToAppend.Add(
                            DataAccess.CreateParameterDouble( Convert.ToDouble( value1.Trim() ) ) );
                        requireAnd = true;
                    }

                    if (value2 != null && value2.Trim() != String.Empty)
                    {
                        if (requireAnd)
                            command += "AND ";

                        command += String.Format( " {0} <= @{1} ", sqlField, fieldName + "1" );
                        dbParametersToAppend.Add(
                            DataAccess.CreateParameterDouble( Convert.ToDouble( value2.Trim() ) ) );
                    }
                    break;

                case SearchFilterType.Date:
                    if (value1 != null && value1.Trim() != String.Empty)
                    {
                        command += String.Format( " {0} >= @{1} ", sqlField, fieldName );

                        DateTime dateOrig = Convert.ToDateTime( value1.Trim() );
                        dbParametersToAppend.Add(
                            DataAccess.CreateParameterDateTime( DateTimeUtilities.StartTimeOfDay( dateOrig ) ) );

                        requireAnd = true;
                    }

                    if (value2 != null && value2.Trim() != String.Empty)
                    {
                        if (requireAnd)
                            command += "AND ";

                        DateTime dateOrig = Convert.ToDateTime( value2.Trim() );

                        command += String.Format( " {0} <= @{1} ", sqlField, fieldName + "1" );
                        dbParametersToAppend.Add(
                            DataAccess.CreateParameterDateTime( DateTimeUtilities.EndTimeOfDay( dateOrig ) ) );
                    }
                    break;
            }

            return command;
        }

        private static string CreateExtraFilter( string extraFilter )
        {
            if (!String.IsNullOrEmpty( extraFilter ))
                return " AND " + extraFilter + " ";
            else
                return String.Empty;
        }

        private static string ReplaceSpecialCharecter( string str )
        {
            string[] allString = str.Split( '[' );
            for (int i = 0; i < allString.Length; i++)
            {
                allString[i] = allString[i].Replace( "]", "[]]" );
            }
            return string.Join( "[[]", allString );
        }

        private static string RevertField( string fieldName )
        {
            string[] fieldNames = fieldName.Split( '.' );
            return fieldNames[fieldNames.Length - 1];
        }

        public static string CreateCommandWithAnd(
            SearchFilterType filterType,
            string fieldName,
            string value1,
            string value2,
            ArrayList dbParametersToAppend
            )
        {
            string command = CreateCommandNoPrefix( filterType, fieldName, String.Empty, value1, value2, dbParametersToAppend );
            if (!String.IsNullOrEmpty( command ))
                return " AND " + command;
            else
                return String.Empty;
        }

        public static string CreateCommandWithWhere(
            SearchFilterType filterType,
            string fieldName,
            string value1,
            string value2,
            ArrayList dbParametersToAppend
            )
        {
            return CreateCommandWithWhere( filterType, fieldName, value1, value2, String.Empty, dbParametersToAppend );
        }

        public static string CreateCommandWithWhere(
            SearchFilterType filterType,
            string fieldName,
            string value1,
            string value2,
            string extrafilter,
            ArrayList dbParametersToAppend
            )
        {
            return CreateCommandWithWhere( filterType, RevertField( fieldName ), fieldName, value1, value2, extrafilter, dbParametersToAppend );
        }

        public static string CreateCommandWithWhere(
            SearchFilterType filterType,
            string fieldName,
            string sqlField,
            string value1,
            string value2,
            string extrafilter,
            ArrayList dbParametersToAppend
            )
        {
            string command = CreateCommandNoPrefix( filterType, fieldName, sqlField, value1, value2, dbParametersToAppend );
            if (!String.IsNullOrEmpty( command ))
                return " WHERE " + command + CreateExtraFilter( extrafilter );
            else
                return " WHERE 1 = 1 " + CreateExtraFilter( extrafilter );
        }

    }
}
