﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.Utilities;
using System.Web.UI;
using Vevo.Domain;
using Vevo.Base.Data.Caching;
using Vevo.Domain.GoogleFeed;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class GoogleFeedMappingRepository : IGoogleFeedMappingRepository
    {
        private const string RepositoryName = "GoogleFeedOptionMappingRepository";
        private const string RepositoryName1 = "GoogleFeedSpecMappingRepository";

        private const string GetGoogleOptionMappingTableSchemaSignature = "GetGoogleOptionMappingTableSchema";
        private const string GetGoogleSpecMappingTableSchemaSignature = "GetGoogleSpecMappingTableSchema";
        private const string GetOneGoogleOptionMappingSignature = "GetOneGoogleOptionMapping";
        private const string GetOneGoogleSpecMappingSignature = "GetOneGoogleSpecMapping";
        private const string GetAllGoogleOptionMappingSignature = "GetAllGoogleOptionMapping";
        private const string GetAllGoogleSpecMappingSignature = "GetAllGoogleSpecMapping";
        private const string SearchGoogleOptionMappingSignature = "SearchGoogleOptionMapping";
        private const string SearchGoogleSpecMappingSignature = "SearchGoogleSpecMapping";
        
        private ICacheDependencyUtility _cacheDependency
            = new CacheDependencyUtility( new string[] { 
                "GoogleFeedTag",
                "GoogleFeedTagValue",
                "GoogleOptionMapping",
                "GoogleSpecMapping",
                "SpecificationGroup",
                "SpecificationGroupLocale",
                "SpecificationItem",
                "SpecificationItemLocale",
                "OptionGroup", 
                "OptionGroupLocale"
                 } );

        private DataCache _dataCache = new DataCache( RepositoryName );
        private DataCache _dataCache1 = new DataCache( RepositoryName1 );
        private GoogleFeedMappingRepositoryNoCache _repository = new GoogleFeedMappingRepositoryNoCache();

        #region Constructors

        public GoogleFeedMappingRepository()
        {
        }

        public GoogleFeedMappingRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region GoogleFeedTagRepository Members

        public IList<TableSchemaItem> GetGoogleOptionMappingTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                   GetGoogleOptionMappingTableSchemaSignature, _cacheDependency
                   );

            if (list == null)
            {
                list = _repository.GetGoogleOptionMappingTableSchema();
                _dataCache.SetValue(
                    GetGoogleOptionMappingTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        public IList<TableSchemaItem> GetGoogleSpecMappingTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                   GetGoogleSpecMappingTableSchemaSignature, _cacheDependency
                   );

            if (list == null)
            {
                list = _repository.GetGoogleSpecMappingTableSchema();
                _dataCache.SetValue(
                    GetGoogleSpecMappingTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        public GoogleOptionMapping SaveGoogleOptionMapping( Culture culture, GoogleOptionMapping googleOptionMapping )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            GoogleOptionMapping result = _repository.SaveGoogleOptionMapping( culture, googleOptionMapping );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public GoogleSpecMapping SaveGoogleSpecMapping( Culture culture, GoogleSpecMapping googleSpecMapping )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            GoogleSpecMapping result = _repository.SaveGoogleSpecMapping( culture, googleSpecMapping );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public GoogleOptionMapping UpdateGoogleOptionMapping( Culture culture, string oldGoogleFeedTagID, string oldOptionGroupID, GoogleOptionMapping newGoogleOptionMapping )
        {
            GoogleOptionMapping result = _repository.UpdateGoogleOptionMapping( culture, oldGoogleFeedTagID, oldOptionGroupID, newGoogleOptionMapping );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public GoogleSpecMapping UpdateGoogleSpecMapping( Culture culture, string oldTagValueID, string oldSpecValue, GoogleSpecMapping newGoogleSpecMapping )
        {
            GoogleSpecMapping result = _repository.UpdateGoogleSpecMapping( culture, oldTagValueID, oldSpecValue, newGoogleSpecMapping );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public GoogleOptionMapping GetOneGoogleOptionMapping( Culture culture, string googleFeedTagID, string optionGroupID )
        {
            GoogleOptionMapping result = (GoogleOptionMapping) _dataCache.GetValue(
                 GetOneGoogleOptionMappingSignature, _cacheDependency, culture.CultureID, googleFeedTagID, optionGroupID );

            if (result == null)
            {
                result = new GoogleOptionMapping();
                result = _repository.GetOneGoogleOptionMapping( culture, googleFeedTagID, optionGroupID );
                _dataCache.SetValue(
                    GetOneGoogleOptionMappingSignature,
                    result,
                    culture.CultureID,
                    googleFeedTagID,
                    optionGroupID
                    );
            }
            return result;
        }

        public GoogleSpecMapping GetOneGoogleSpecMapping( Culture culture, string googleFeedTagValueID, string specificationItemValue )
        {
            GoogleSpecMapping result = (GoogleSpecMapping) _dataCache.GetValue(
                 GetOneGoogleSpecMappingSignature, _cacheDependency, culture.CultureID, googleFeedTagValueID, specificationItemValue );

            if (result == null)
            {
                result = new GoogleSpecMapping();
                result = _repository.GetOneGoogleSpecMapping( culture, googleFeedTagValueID, specificationItemValue );
                _dataCache.SetValue(
                    GetOneGoogleSpecMappingSignature,
                    result,
                    culture.CultureID,
                    googleFeedTagValueID,
                    specificationItemValue
                    );
            }
            return result;
        }

        public IList<GoogleOptionMapping> GetAllGoogleOptionMapping( Culture culture )
        {
            IList<GoogleOptionMapping> result = (IList<GoogleOptionMapping>) _dataCache.GetValue(
                 GetAllGoogleOptionMappingSignature, _cacheDependency, culture.CultureID );

            if (result == null)
            {
                result = new List<GoogleOptionMapping>();
                result = _repository.GetAllGoogleOptionMapping( culture );
                _dataCache.SetValue(
                    GetAllGoogleOptionMappingSignature,
                    result,
                    culture.CultureID
                    );
            }
            return result;
        }

        public IList<GoogleSpecMapping> GetAllGoogleSpecMapping( Culture culture )
        {
            IList<GoogleSpecMapping> result = (IList<GoogleSpecMapping>) _dataCache.GetValue(
                 GetAllGoogleSpecMappingSignature, _cacheDependency, culture.CultureID );

            if (result == null)
            {
                result = new List<GoogleSpecMapping>();
                result = _repository.GetAllGoogleSpecMapping( culture );
                _dataCache.SetValue(
                    GetAllGoogleSpecMappingSignature,
                    result,
                    culture.CultureID
                    );
            }
            return result;
        }

        public IList<GoogleOptionMapping> SearchGoogleOptionMapping(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchGoogleOptionMappingSignature,
                _cacheDependency,
                sortBy,
                culture.CultureID,
                searchFilter.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchGoogleOptionMapping( culture, sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchGoogleOptionMappingSignature,
                    result,
                    culture.CultureID,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<GoogleOptionMapping>) result.First;
        }

        public IList<GoogleSpecMapping> SearchGoogleSpecMapping(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchGoogleSpecMappingSignature,
                _cacheDependency,
                culture.CultureID,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchGoogleSpecMapping( culture, sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchGoogleSpecMappingSignature,
                    result,
                    culture.CultureID,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<GoogleSpecMapping>) result.First;
        }

        public void DeleteGoogleOptionMapping( string googleFeedTagID, string optionGroupID )
        {
            _repository.DeleteGoogleOptionMapping( googleFeedTagID, optionGroupID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteGoogleSpecMapping( string googleFeedTagValueID, string specificationValue )
        {
            _repository.DeleteGoogleSpecMapping( googleFeedTagValueID, specificationValue );
            _dataCache.ClearRepositoryCache();
        }

        #endregion
    }
}
