﻿using System;
using System.Collections;
using System.Data;
using System.Data.Common;
using System.Configuration;
using System.Text;
using Vevo;
using Vevo.Domain;
using Vevo.Domain.Products;
using Vevo.Shared.DataAccess;

namespace Vevo.Data.Implementation
{
    internal class ProductCustomPriceAccess
    {
        private const string AllProductCustomPriceFields =
           "ProductID, DefaultPrice, MinimumPrice";

        public Product Create( Product product )
        {
            //ProductCustomPrice product = (ProductCustomPrice) productCustomPrice;

            DataAccess.ExecuteNonQuery(
                "INSERT INTO ProductCustomPrice " +
                " (" + AllProductCustomPriceFields + ") " +
                "VALUES (@ProductID, @DefaultPrice, @MinimumPrice);",
                DataAccess.CreateParameterString( product.ProductID ),
                DataAccess.CreateParameterDecimal( product.ProductCustomPrice.DefaultPrice ),
                DataAccess.CreateParameterDecimal( product.ProductCustomPrice.MinimumPrice )
            );

            return product;
        }

        public Product Update( Product product )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE ProductCustomPrice " +
                "SET DefaultPrice = @DefaultPrice, " +
                "MinimumPrice = @MinimumPrice " +
                "WHERE ProductID = @ProductID;",
                DataAccess.CreateParameterDecimal( product.ProductCustomPrice.DefaultPrice ),
                DataAccess.CreateParameterDecimal( product.ProductCustomPrice.MinimumPrice ),
                DataAccess.CreateParameterString( product.ProductID )
            );

            return product;
        }

        public void DeleteByProductID( string productID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM ProductCustomPrice " +
                "WHERE ProductID = @ProductID;",
                DataAccess.CreateParameterString( productID )
            );
        }

        public Hashtable GetDetails( string productID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllProductCustomPriceFields +
                "FROM ProductCustomPrice " +
                "WHERE ProductID = @ProductID;",
                DataAccess.CreateParameterString( productID )
            );

            if (table.Rows.Count > 0)
                return DataAccessConverter.ConvertDataRowToHashtable( table.Rows[0] );
            else
                return null;
        }
        public bool IsExistItem( string productID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT ProductID " +
                "FROM ProductCustomPrice " +
                "WHERE ProductID = @ProductID;",
                DataAccess.CreateParameterString( productID )
            );

            if (table.Rows.Count > 0)
                return true;
            else
                return false;
        }
        
    }
}
