﻿using System.Collections.Generic;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Returns;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class RmaActionRepository : IRmaActionRepository
    {
        #region Private

        private const string RepositoryName = "RmaActionRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetAllSignature = "GetAll";
        private const string SearchRmaActionSignature = "SearchRmaAction";
        private const string GetTableSchemaSignature = "GetTableSchema";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( "RmaAction" );
        private RmaActionRepositoryNoCache _repository = new RmaActionRepositoryNoCache();

        #endregion

        #region Constructors

        public RmaActionRepository()
        {

        }

        public RmaActionRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public RmaAction Save( RmaAction rmaAction )
        {
            RmaAction newRmaAction = _repository.Save( rmaAction );
            _dataCache.ClearRepositoryCache();
            return newRmaAction;
        }

        public RmaAction GetOne( string rmaActionID )
        {
            RmaAction result = ( RmaAction ) _dataCache.GetValue( GetOneSignature, _cacheDependency, rmaActionID );

            if ( result == null )
            {
                result = _repository.GetOne( rmaActionID );
                _dataCache.SetValue( GetOneSignature, result, rmaActionID );
            }

            return result;
        }

        public IList<RmaAction> GetAll( BoolFilter isEnabled )
        {
            IList<RmaAction> list = ( IList<RmaAction> ) _dataCache.GetValue( GetAllSignature, _cacheDependency, isEnabled );

            if ( list == null )
            {
                list = _repository.GetAll( isEnabled );

                _dataCache.SetValue( GetAllSignature, list, isEnabled );
            }
            return list;
        }

        public void Delete( string rmaActionID )
        {
            _repository.Delete( rmaActionID );
            _dataCache.ClearRepositoryCache();
        }

        public IList<RmaAction> SearchRmaAction( string sortBy, SearchFilter searchFilter )
        {
            Pair result = ( Pair ) _dataCache.GetValue(
                SearchRmaActionSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString()
                );

            if ( result == null )
            {
                result = new Pair();
                result.First = _repository.SearchRmaAction( sortBy, searchFilter );
                _dataCache.SetValue(
                    SearchRmaActionSignature,
                    result,
                    sortBy,
                    searchFilter.ToString()
                    );
            }

            return ( IList<RmaAction> ) result.First;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = ( IList<TableSchemaItem> ) _dataCache.GetValue( GetTableSchemaSignature, _cacheDependency );

            if ( list == null )
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue( GetTableSchemaSignature, list );
            }

            return list;
        }

        #endregion
    }
}
