using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using PayPal.Payments.DataObjects;
using PayPal.Payments.Transactions;
using PayPal.Payments.Common.Utility;
using com.paypal.sdk.services;
using com.paypal.sdk.util;

using Vevo.Domain;
using Vevo.Domain.Configurations;
using Vevo.Domain.Orders;
using Vevo.WebAppLib;

namespace Vevo.Domain.Payments.PayPalProExpress
{
    /// <summary>
    /// Summary description for PayPalProUKPayment
    /// </summary>
    public class PayPalProExpressPaymentMethod : AnonymousPaymentMethod
    {
        private NVPCodec _decoder;

        public PayPalProExpressPaymentMethod()
            : base()
        {
        }

        public override string Name
        {
            get
            {
                return PaymentOption.PayPalProExpress;
            }
        }

        public override bool ProcessPayment(
            decimal amount,
            string currency,
            CheckoutDetails checkout,
            out ProcessPaymentResult result )
        {
            NVPCallerServices caller = PayPalProExpressUtilities.Instance.PayPalAPIInitialize();
            NVPCodec encoder = new NVPCodec();
            encoder["METHOD"] = "DoExpressCheckoutPayment";
            encoder["TOKEN"] = checkout.CustomParameters[CheckoutDetails.PayPalProExpress_TokenID].ToString();
            encoder["PAYERID"] = checkout.CustomParameters[CheckoutDetails.PayPalProExpress_PayerID].ToString();
            encoder["VERSION"] = "65.0";
            encoder["PAYMENTACTION"] = PayPalProExpressUtilities.PaymentType;
            encoder["CURRENCYCODE"] = currency;
            encoder["AMT"] = amount.ToString( "f2" );

            
            string pStrrequestforNvp = encoder.Encode();
            string pStresponsenvp = caller.Call( pStrrequestforNvp );

            NVPCodec decoder = new NVPCodec();
            decoder.Decode( pStresponsenvp );


            string strAck = decoder["ACK"];
            if (strAck != null && (strAck == "Success" || strAck == "SuccessWithWarning"))
            {
                result = new ProcessPaymentResult(
                    decoder["TRANSACTIONID"],
                    String.Empty,
                    pStresponsenvp );

                return true;
            }
            else
            {
                result = new ProcessPaymentResult(
                    String.Empty,
                    PayPalProExpressUtilities.GetErrorMessage( decoder ),
                    String.Empty );

                return false;
            }
        }

        public bool ProcessPostLoginDetails( string token, CheckoutDetails checkoutDetails )
        {
            NVPCallerServices caller = PayPalProExpressUtilities.Instance.PayPalAPIInitialize();
            NVPCodec encoder = new NVPCodec();
            encoder["METHOD"] = "GetExpressCheckoutDetails";
            encoder["TOKEN"] = token;
            //checkoutSession.CheckoutDetails.CustomParameters[CheckoutDetails.PayPalProExpress_TokenID] = token;
            checkoutDetails.CustomParameters[CheckoutDetails.PayPalProExpress_TokenID] = token;

            string pStrrequestforNvp = encoder.Encode();
            string pStresponsenvp = caller.Call( pStrrequestforNvp );

            NVPCodec decoder = new NVPCodec();
            decoder.Decode( pStresponsenvp );

            _decoder = decoder;

            string strAck = decoder["ACK"];
            if (strAck != null && (strAck == "Success" || strAck == "SuccessWithWarning"))
            {
                checkoutDetails.CustomParameters[CheckoutDetails.PayPalProExpress_PayerID] = decoder["PAYERID"];
                Vevo.Base.Domain.Address billingAddress = new Vevo.Base.Domain.Address( decoder["FIRSTNAME"],
                    decoder["LASTNAME"],
                    String.Empty,
                    decoder["SHIPTOSTREET"],
                    String.Empty,
                    decoder["SHIPTOCITY"],
                    decoder["SHIPTOSTATE"],
                    decoder["SHIPTOZIP"],
                    decoder["SHIPTOCOUNTRYCODE"],
                    String.Empty,
                    String.Empty );

                checkoutDetails.BillingAddress = billingAddress;
                checkoutDetails.Email = decoder["EMAIL"];

                Vevo.Base.Domain.Address shippingAddress = new Vevo.Base.Domain.Address(
                    decoder["FIRSTNAME"],
                    decoder["LASTNAME"],
                    String.Empty,
                    decoder["SHIPTOSTREET"],
                    String.Empty,
                    decoder["SHIPTOCITY"],
                    decoder["SHIPTOSTATE"],
                    decoder["SHIPTOZIP"],
                    decoder["SHIPTOCOUNTRYCODE"],
                    String.Empty,
                    String.Empty
                    );

                checkoutDetails.ShippingAddress =
                    new Vevo.Base.Domain.ShippingAddress( shippingAddress, false );

                return true;
            }
            else
            {
                return false;
            }
        }

        public string GetErrorMessage()
        {
            return PayPalProExpressUtilities.GetErrorMessage( _decoder );
        }
    }
}