﻿using System;
using System.Data;
using System.Configuration;

using Vevo.Domain.Products;

namespace Vevo.Domain.Orders
{
    [Serializable]
    public class ProductKitItemValue
    {
        #region Private
        private ProductKitItem _kitItem;
        private ProductKitGroup.ProductKitGroupType _groupType;
        private string _details;
        private int _quantity;

        private void Initialize(
            ProductKitItem item,
            ProductKitGroup.ProductKitGroupType groupType,
            string details,
            int qty)
        {
            _kitItem = item;
            _groupType = groupType;
            _details = details;
            _quantity = qty;
        }
        #endregion

        #region Constructors
        public ProductKitItemValue(
            ProductKitItem item,
            ProductKitGroup.ProductKitGroupType groupType,
            string details,
            int qty )
        {
            Initialize( item, groupType, details ,qty);
        }

        #endregion

        #region Public Properties
        public ProductKitItem ProductKitItem
        {
            get { return _kitItem; }
        }

        public ProductKitGroup.ProductKitGroupType GroupType
        {
            get { return _groupType; }
        }

        public string ProductKitGroupID
        {
            get { return _kitItem.ProductKitGroupID; }
        }

        public string ProductID
        {
            get { return _kitItem.ProductID; }
        }

        public bool IsDefinedQuantity
        {
            get { return _kitItem.IsUserDefinedQuantity; }
        }

        public int Quantity
        {
            get 
            {
                if (!IsDefinedQuantity)
                    return _kitItem.Quantity;
                else
                    return _quantity; 
            }
        }

        public string Details
        {
            get { return _details; }
        }
        #endregion

        #region Public Methods

        public string GetDisplayedName(
            Culture culture,
            string storeID)
        {
            Product product = DataAccessContext.ProductRepository.GetOne( culture, _kitItem.ProductID, storeID );
            return product.Name + " x" + Quantity.ToString();
            
        }

        public string GetGroupName(
                     Culture culture,
            string storeID)
        {
            ProductKitGroup group = DataAccessContext.ProductKitGroupRepository.GetOne( culture, ProductKitGroupID );
            return group.Name;
        }

        #endregion
    }
}
