using System;
using System.Collections.Generic;
using System.Text;
using Vevo.Domain.Base;
using Vevo.Domain.Users;
using System.Text.RegularExpressions;
using Vevo.Base.Domain;

namespace Vevo.Domain.Shipping
{
    public class ShippingZoneGroup
    {
        #region Private

        private string _zoneGroupID = "0";
        private string _zoneName = String.Empty;

        private IEntityList<ShippingZoneItem> _zoneItem;

        private bool IsCountryExisting( ShippingZoneItem item, string country )
        {
            if (!String.IsNullOrEmpty( item.CountryCode ))
                return item.CountryCode == country;
            else
                return true;
        }

        private bool IsStateExisting( ShippingZoneItem item, string state )
        {
            if (!String.IsNullOrEmpty( item.StateCode ))
                return item.StateCode == state;
            else
                return true;
        }

        private bool IsZipCodeExisting( ShippingZoneItem item, string zipCode )
        {
            if (!String.IsNullOrEmpty( item.ZipCode ))
            {
                string widecart = item.ZipCode.Trim().ToLower().Replace( "*", ".*" );
                Regex r = new Regex( '^' + widecart + '$' );
                return r.IsMatch( zipCode.Trim().ToLower() );
            }
            else
                return true;
        }

        #endregion

        #region Public Properties

        public static ShippingZoneGroup Null
        {
            get { return NullShippingZoneGroup.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }

        public string ZoneGroupID { get { return _zoneGroupID; } set { _zoneGroupID = value; } }
        public string ZoneName { get { return _zoneName; } set { _zoneName = value; } }


        public IEntityList<ShippingZoneItem> ZoneItem
        {
            get
            {
                if (_zoneItem == null)
                    _zoneItem = new EntityList<ShippingZoneItem>(
                        DataAccessContext.ShippingZoneGroupRepository.GetShippingZoneItemByZoneGroupID( ZoneGroupID ) );

                return _zoneItem;
            }
        }

        public bool IsAddressMatch( ShippingAddress shippingAddress )
        {
            foreach (ShippingZoneItem item in ZoneItem)
            {
                if (IsCountryExisting( item, shippingAddress.Country )
                    && IsStateExisting( item, shippingAddress.State )
                    && IsZipCodeExisting( item, shippingAddress.Zip ))
                {

                    return true;
                }
            }
            return false;
        }

        #endregion
    }
}
