using System;
using System.Collections.Generic;
using System.Text;

using Vevo.Domain.Configurations;
using Vevo.Domain.Discounts;

namespace Vevo.Domain.Users
{
    public class WholesaleStatus
    {
        #region Public Enum

        public enum DiscountType
        {
            FixedPrice = 1,
            Percentage = 2
        }

        #endregion


        #region Private

        private ConfigurationCollection _configurations;

        private int _level;


        private WholesaleStatus.DiscountType GetWholesaleMode()
        {
            if (_configurations.GetIntValue( "WholesaleMode" ) == 1)
                return WholesaleStatus.DiscountType.FixedPrice;
            else
                return WholesaleStatus.DiscountType.Percentage;
        }

        #endregion


        #region Constructors

        public WholesaleStatus(
            ConfigurationCollection configurations,
            int level )
        {
            _configurations = configurations;
            _level = level;
        }

        public WholesaleStatus( int level )
        {
            _configurations = DataAccessContext.Configurations;
            _level = level;
        }

        #endregion


        #region Public Properties

        public int Level { get { return _level; } }

        // Return null object
        public static WholesaleStatus Null
        {
            get { return NullWholesaleStatus.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }

        #endregion


        #region Public Methods

        public virtual decimal GetPricePercentageOnly(
            decimal price )
        {
            return GetPrice( price, price, price, price );
        }

        public virtual decimal GetPrice(
            decimal price,
            decimal wholesalePrice,
            decimal wholesalePrice2,
            decimal wholesalePrice3 )
        {
            if (GetWholesaleMode() == DiscountType.FixedPrice)
            {
                switch (_level)
                {
                    case 1:
                        return wholesalePrice;
                    case 2:
                        return wholesalePrice2;
                    case 3:
                        return wholesalePrice3;
                    default:
                        return price;
                }
            }
            else
            {
                switch (_level)
                {
                    case 1:
                        return price * (decimal) (100 - _configurations.GetDoubleValue( "DiscountLevel1" )) / 100;
                    case 2:
                        return price * (decimal) (100 - _configurations.GetDoubleValue( "DiscountLevel2" )) / 100;
                    case 3:
                        return price * (decimal) (100 - _configurations.GetDoubleValue( "DiscountLevel3" )) / 100;
                    default:
                        return price;
                }
            }
        }

        #endregion

    }
}
