﻿using System;
using System.Collections.Generic;
using Vevo.Base.Domain;
using Vevo.Deluxe.Domain;
using Vevo.Deluxe.Domain.Users;
using Vevo.Domain;
using Vevo.Domain.Users;
using Vevo.WebAppLib;
using Vevo.WebServices.SearchFilters;
using Vevo.WebServices.Security;

namespace Vevo.WebServices.Customers
{
    public class CustomerService : ICustomerService
    {
        private CustomerConstraints ConvertCustomerToConstraints( Customer customer )
        {
            CustomerConstraints constraints = new CustomerConstraints();
            constraints.AffiliateCode = customer.AffiliateCode;
            constraints.AffiliateExpireDate = customer.AffiliateExpireDate;
            constraints.CustomerID = customer.CustomerID;
            constraints.Email = customer.Email;
            constraints.IsEnabled = customer.IsEnabled;
            constraints.IsQBExported = customer.IsQBExported;
            constraints.IsWholesale = customer.IsWholesale;
            constraints.MerchantNotes = customer.MerchantNotes;
            constraints.QBListID = customer.QBListID;
            constraints.RegisterDate = customer.RegisterDate;
            constraints.UseBillingAsShipping = customer.UseBillingAsShipping;
            constraints.UserName = customer.UserName;
            constraints.WholesaleLevel = customer.WholesaleLevel;

            constraints.IsTaxExempt = customer.IsTaxExempt;
            constraints.TaxExemptID = customer.TaxExemptID;
            constraints.TaxExemptCountry = customer.TaxExemptCountry;
            constraints.TaxExemptState = customer.TaxExemptState;

            constraints.BillingAddress = new AddressConstraints();
            constraints.BillingAddress.Address1 = customer.BillingAddress.Address1;
            constraints.BillingAddress.Address2 = customer.BillingAddress.Address2;
            constraints.BillingAddress.City = customer.BillingAddress.City;
            constraints.BillingAddress.Company = customer.BillingAddress.Company;
            constraints.BillingAddress.Country = customer.BillingAddress.Country;
            constraints.BillingAddress.Fax = customer.BillingAddress.Fax;
            constraints.BillingAddress.FirstName = customer.BillingAddress.FirstName;
            constraints.BillingAddress.LastName = customer.BillingAddress.LastName;
            constraints.BillingAddress.Phone = customer.BillingAddress.Phone;
            constraints.BillingAddress.State = customer.BillingAddress.State;
            constraints.BillingAddress.Zip = customer.BillingAddress.Zip;

            foreach (ShippingAddress shippingAddress in customer.ShippingAddresses)
            {
                ShippingAddressConstraints shippingAddressConstraints = new ShippingAddressConstraints();
                shippingAddressConstraints.AliasName = shippingAddress.AliasName;
                shippingAddressConstraints.Address1 = shippingAddress.Address1;
                shippingAddressConstraints.Address2 = shippingAddress.Address2;
                shippingAddressConstraints.City = shippingAddress.City;
                shippingAddressConstraints.Company = shippingAddress.Company;
                shippingAddressConstraints.Country = shippingAddress.Country;
                shippingAddressConstraints.CustomerID = shippingAddress.CustomerID;
                shippingAddressConstraints.Fax = shippingAddress.Fax;
                shippingAddressConstraints.FirstName = shippingAddress.FirstName;
                shippingAddressConstraints.IsResidential = shippingAddress.Residential;
                shippingAddressConstraints.ShippingAddressID = shippingAddress.ShippingAddressID;
                shippingAddressConstraints.IsSameAsBillingAddress = shippingAddress.IsSameAsBillingAddress;
                shippingAddressConstraints.LastName = shippingAddress.LastName;
                shippingAddressConstraints.Phone = shippingAddress.Phone;
                shippingAddressConstraints.State = shippingAddress.State;
                shippingAddressConstraints.Zip = shippingAddress.Zip;

                if (constraints.ShippingAddress == null)
                    constraints.ShippingAddress = new List<ShippingAddressConstraints>();
                constraints.ShippingAddress.Add( shippingAddressConstraints );
            }

            foreach (CustomerSubscription customerSubscription in DataAccessContextDeluxe.CustomerSubscriptionRepository.GetCustomerSubscriptionsByCustomerID( customer.CustomerID ))
            {
                CustomerSubscriptionConstraints customerSubscriptionConstraints = new CustomerSubscriptionConstraints();
                customerSubscriptionConstraints.CustomerID = customer.CustomerID;
                customerSubscriptionConstraints.SubscriptionLevelID = customerSubscription.SubscriptionLevelID;
                customerSubscriptionConstraints.StartDate = customerSubscription.StartDate;
                customerSubscriptionConstraints.EndDate = customerSubscription.EndDate;
                customerSubscriptionConstraints.IsActive = customerSubscription.IsActive;
                if (constraints.CustomerSubscriptions == null)
                    constraints.CustomerSubscriptions = new List<CustomerSubscriptionConstraints>();
                constraints.CustomerSubscriptions.Add( customerSubscriptionConstraints );

            }

            return constraints;
        }

        private CustomerListResponseMessage CustomListCustomers( CustomerListRequestMessage ListMessage, BoolFilter boolFiler )
        {
            CustomerListResponseMessage responseMessage = new CustomerListResponseMessage();
            if (WcfSessionPool.GetSession( ListMessage.Ticket ) == null)
            {
                responseMessage.ErrorMessage = "Error: Authentication is failed.";
                return responseMessage;
            }
            try
            {
                SearchFilter searchFilter = SearchFilterUtilities.ConvertConstraintToSearchFilter(
                                ListMessage.SearchFilter );
                int customerCount = 0;
                IList<Customer> customer_result = DataAccessContext.CustomerRepository.SearchCustomer( "CustomerID", searchFilter, boolFiler, ListMessage.SearchFilter.StartIndex, ListMessage.SearchFilter.EndIndex, out customerCount );
                responseMessage.Customers = new List<CustomerConstraints>();
                foreach (Customer customer in customer_result)
                {
                    responseMessage.Customers.Add( ConvertCustomerToConstraints( customer ) );
                }
            }
            catch (Exception e)
            {
                WebUtilities.LogError( e );
                responseMessage.ErrorMessage = "Error: Cannot get customer data from repository";
            }

            return responseMessage;
        }

        public CustomerListResponseMessage ListCustomers( CustomerListRequestMessage ListMessage )
        {
            return CustomListCustomers( ListMessage, BoolFilter.ShowAll );
        }

        public CustomerListResponseMessage ListWholesales( CustomerListRequestMessage ListMessage )
        {
            return CustomListCustomers( ListMessage, BoolFilter.ShowTrue );
        }
    }
}
