using System;
using System.Data;
using System.Configuration;
using Vevo.DataAccessLib.Cart;
using Vevo.Shared.Utilities;
using Vevo.WebAppLib;
using Vevo.WebUI.DataAccessCache;

namespace Vevo.WebUI.Users
{
    /// <summary>
    /// Summary description for AdminPermission
    /// </summary>
    public class AdminPermission
    {
        private string _adminID;

        // File names with no extension
        private string[] _alwaysAccessiblePages = new string[] { 
            "Default", 
            "GenericError",
            "AdminError",
            "AdminMessage",
            "AdminConfig",
            "SecurityWarning",
            "Help"
        };


        private string RemoveFileExtension( string fileName )
        {
            int index = fileName.IndexOf( '.' );
            if (index == -1)
            {
                return fileName;
            }
            else
            {
                return fileName.Substring( 0, index );
            }
        }

        private string GetPageName( string url )
        {
            string[] splitURL = url.Split( '/' );
            int upperBound = splitURL.GetUpperBound( 0 );

            string pageNameLong = splitURL[upperBound].ToString().ToLower();

            string[] splitParam = pageNameLong.Split( '?' );
            return splitParam[0];
        }

        private bool PageRequiresPermission( string url )
        {
            foreach (string page in _alwaysAccessiblePages)
            {
                if (String.Compare(
                    page,
                    RemoveFileExtension( GetPageName( url ) ),
                    true ) == 0)
                {
                    return false;
                }
            }

            return true;
        }

        private bool GetPermissionByMenuPageName( string menuPageName, string attributeName )
        {
            DataTable table = AdminMenuPermissionAccessCache.Instance.GetAllByAdminID( _adminID );
            DataTable pageTable = AdminMenuPageAccessCache.Instance.GetAll( CultureIDs.English );

            menuPageName = menuPageName.Replace( ".aspx", ".ascx" );

            DataRow[] rows = table.Select( String.Format( "MenuPageName = '{0}'", menuPageName ) );
            if (rows.Length > 0)
                return ConvertUtilities.ToBoolean( rows[0][attributeName] );
            else
            {
                DataRow[] subrows = pageTable.Select( String.Format( "MenuPageName = '{0}'", menuPageName ) );
                if (subrows.Length > 0)
                    return GetPermissionByMenuPageName( subrows[0]["ParentMenuPageName"].ToString(), attributeName );
                else
                    return false;
            }
        }

        private bool GetPermissionAttribute( string url, string attributeName )
        {
            if (PageRequiresPermission( url ))
            {
                return GetPermissionByMenuPageName(
                    GetPageName( url ),
                    attributeName );
            }
            else
            {
                return true;
            }
        }


        public AdminPermission( string adminID )
        {
            _adminID = adminID;
        }

        public bool CanViewPage( string url )
        {
            return GetPermissionAttribute( url, "ViewMode" );
        }

        public bool CanModifyPage( string url )
        {
            return GetPermissionAttribute( url, "ModifyMode" );
        }

    }
}
